#=============================================================================
# Copyright (C) 2003           Ikki Ohmukai,      All rights reserved.
#
#
# This file is part of RNA.
#
#    RNA is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RNA is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RNA; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#=============================================================================

#=============================================================================
# ユーティリティ
#=============================================================================
package main;
use LWP::UserAgent;
use strict;


# 2003-05-10T22:57:49 
# $str = &dc_date($year, $month, $day, $hour, $minute, $second, $zone)
sub dc_date
{
	my ($year, $month, $day, $hour, $minute, $second, $zone) = @_;
	return sprintf("%4d-%02d-%02dT%02d:%02d:%02d%s",
		$year,$month,$day,$hour,$minute,$second, $zone);
}

# $dc_date = &rfc822date_to_dc($rfc822date);
#
# http://asg.web.cmu.edu/rfc/rfc822.html#sec-5
#    e.g.)	Sun, 29 Sep 2002 19:59:01 GMT
#
sub rfc822date_to_dc
{
	my $date = shift;
	chomp($date);
	
	my %month_hash = (
		Jan => '01', Feb=>'02', Mar=>'03', Apr=>'04', May=>'05', Jun=>'06', 
		Jul=>'07', Aug=> '08', Sep=>'09', Oct=>'10', Nov=>'11', Dec=>'12');

	my ($wd_str, $ymdhmsz) = split(/\s*,\s*/, $date);
	unless($ymdhmsz) {
		$ymdhmsz = $wd_str;
	}
	my ($day, $m_str, $year, $time, $zone) = split(/\s+/, $ymdhmsz);
	my $month = $month_hash{$m_str};

	# 00:00:00
	my ($h, $m, $s) = split(/:/, $time);

	# zone
	my %zone_hash = (UT=>'', GMT=>'', EST=>'-05:00', EDT=>'-04:00', 
		CST=>'-06:00', CDT=>'-05:00', MST=>'-07:00', MDT=>'-06:00', 
		PST=>'-08:00', PDT=>'-07:00', 
		Z=>'', A=>'-01:00', , B=>'-02:00', C=>'-03:00', D=>'-04:00', 
		E=>'-05:00', F=>'-06:00', G=>'-07:00', H=>'-08:00', I=>'-09:00', 
		K=>'-10:00', L=>'-11:00', M=>'-12:00', N=>'+01:00', O=>'+02:00', 
		P=>'+03:00', Q=>'+04:00', R=>'+05:00', S=>'+06:00', T=>'+07:00', 
		U=>'+08:00', V=>'+09:00', W=>'+10:00', X=>'+11:00', Y=>'+12:00');

	my $zones = join('|', keys %zone_hash);
	my $z = '';
	if($zone =~ /^($zones)$/) {
		$z = $zone_hash{$1};
	}
	elsif($zone =~ /(\-|\+)(\d{2})(\d{2})/) {
		$z = $1 . $2 . ':' . $3;
	}

	return sprintf("%4d-%02d-%02dT%02d:%02d:%02d%s",
		$year,$month,$day,$h,$m,$s, $z);
}

sub dc2ymd
{
	my $dc = shift;

	unless($dc) {
		return '0';
	}

	my ($ymd, $time) = split(/T/, $dc);
	my ($y, $M, $d) = split(/-/, $ymd);
	$time =~ /(\d+):(\d+):(\d+)(.*)/i;
	my ($h, $m, $s, $z) = ($1, $2, $3, $4);

	my $ret_value = sprintf("%4d%02d%02d%02d%02d%02d", $y, $M, $d, $h, $m, $s);
	

	# タイムゾーンに関して
	if($z =~ /([-+])(\d+):(\d+)/i){
		my ($zf, $zh, $zm) = ($1, $2, $3);

		my $diff = sprintf("%02d%02d00", $zh, $zm);
		if($zf eq '-') {
			$ret_value += $diff;
		}
		else {
			$ret_value -= $diff;
		}
	}

	return $ret_value;
#	return ($y, $M, $d, $h, $m, $s);
}

# 2003-05-10T22:57:49+09:00
# $nday = &diff_dc_date($dc_date1, $dc_date2);
# 	$dc_date1 < $dc_date2  :  -
# 	$dc_date1 == $dc_date2 :  0 
# 	$dc_date1 > $dc_date2  :  +, 
sub diff_dc_date
{
	my($dc_date1, $dc_date2) = @_;

#	($y1, $M1, $d1, $h1, $m1, $s1) = &dc2ymd($dc_date1);
#	($y2, $M2, $d2, $h2, $m2, $s2) = &dc2ymd($dc_date2);
#	my $d1 = sprintf("%4d%02d%02d%02d%02d%02d", $y1, $M1, $d1, $h1, $m1, $s1);
#	my $d2 = sprintf("%4d%02d%02d%02d%02d%02d", $y2, $M2, $d2, $h2, $m2, $s2);

	my $d1 = &dc2ymd($dc_date1);
	my $d2 = &dc2ymd($dc_date2);

	return $d1 - $d2;
}

# 2003-05-10T22:57:49+09:00
# $result = &comp_dc_date($dc_date1, $dc_date2);
# 	$dc_date1 < $dc_date2  :  -
# 	$dc_date1 == $dc_date2 :  0 
# 	$dc_date1 > $dc_date2  :  +, 
sub comp_dc_date
{
	my($dc_date1, $dc_date2) = @_;
	return &dc2ymd($dc_date2) <=> &dc2ymd($dc_date1);
}

# &set_rss($rss, $title, $link, $description, $date, $creator, $language);
sub set_rss
{
	my ($rss, $title, $link, $description, $date, $creator, $language) = @_;

	$rss->channel(
		title 		=> $title,
		link		=> $link,
		description	=> $description,
		dc => {
			date       => $date,
			creator    => $creator,
			language   => $language
		},
	);
	
	$rss->add_module(prefix=>'cc', uri=>"http://web.resource.org/cc/");
}

# &print_file($file, \$str);
sub print_file
{
	my ($file, $str_ref) = @_;

	my $str = $$str_ref;
	open(OUT, ">$file") 
		|| die "Content-Type: text/plain\n\nCan't open $file\n";
	flock(OUT, 2);
	print OUT $str;
	flock(OUT, 8);
	close(OUT);
}

# $str = &read_file($file)
sub read_file
{
	my $file = shift;
	
	open(IN, $file) || die "Content-Type: text/plain\n\nCan't open $file\n";
	my $str = "";
	while(my $s = <IN>){
		$str .= $s;
	}

	return $str;
}

# $html = &get_html($uri);
sub get_html
{
	my $uri = shift;

#print $uri . "\n";

	my $req = HTTP::Request->new(GET => $uri);
#	$req->header('Accept' => 'text/html');

	# authorization
#	$req->authorization_basic('user', 'passwd');

	# send request 
	my $ua = LWP::UserAgent->new;
	$ua->agent("RNA" . $ua->agent);
	$ua->parse_head(0); # HTML::HeadParser を必要としないように  
	my $res = $ua->request($req);

	# check
	if ($res->is_success) {
#print $res->content;
		return $res->content;
	}
	else {
#		print "Error: " . $res->status_line . "\n";
		return $res->status_line . "\n";
	}
}


# $result = &html_encode($str)
sub html_encode
{
	my ($t) = @_;
	my %cv = ('&' => '&amp;', '"' => '&quot;',
		'<' => '&lt;',  '>' => '&gt;');
	$t =~ s/([&<>"])/$cv{$1}||$1/ge;
	$t;
}

# $result = &html_decode($str)
sub html_decode 
{
	my ($t) = @_;
	my %cv = ('&amp;' => '&', '&quot;'=> '"',
		'&lt;' => '<', '&gt;' => '>');
	my $s = join(/\|/, keys %cv);
	$t =~ s/($s)/$cv{$1}||$1/ge;
	$t;
}

sub die_handler {
	my $msg = shift;
	print "Content-Type: text/plain\n\n";
	print qq(Error Occured: $msg
	);
	exit(1);
}


# $str = &url_encode($str)
sub url_encode
{
	my $arg = shift;
	$arg =~ s/(\W)/'%'.unpack("H2", $1)/ego;
	$arg =~ tr/ /+/; 

	return $arg;
}

# $str = &url_decode($str)
sub url_decode
{
	my $arg = shift;
	$arg =~ tr/+/ /;
	$arg =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/ego;

	return $arg;
}




1;
__END__

